from PCD.Maths.Vector3D import Vec3


class Feature:
    "Encapsulates a feature, like a window or a gutter"

    def __init__ (self, name, object, just='c', pad=(0,) ):
	"""Constructor

	  name:	  name of the feature [eg. 'door']
	  object: the Object for this feature
	  just:	  the justification for the feature [eg. 't', 'bl', 'c']
	  pad:	  minimum padding: tuple, CSS style: (t [,r [,b [,l]]])
	"""
	self.name = name
	self.object = object
	self.setJustification (just)
	self.setPad (pad)

    def clone (self):
	return Feature (self.name, self.object.clone(), 
				    self.justification, self.pad)
    
    def getJustification (self):
	return self.justification

    def setJustification (self, just):
	assert just in ['t','tl','l','bl','b','br','r','tr','c']
	self.justification = just
    
    def getName (self):
	return self.name

    def setName (self, name):
        self.name = name

    def getObject (self):
	return self.object
	
    def getTPad (self):
	return self.pad[0]
	
    def getRPad (self):
	return self.pad[1]
	
    def getBPad (self):
	return self.pad[2]
	
    def getLPad (self):
	return self.pad[3]
	
    def getHPad (self):
	return self.getLPad() + self.getRPad()
	
    def getVPad (self):
	return self.getTPad() + self.getBPad()

    def setPad (self, pad):
	# make certain we have a tuple of floats
	try:
	    pad = tuple (float (pad))
	except TypeError:
	    pad = tuple ([float(f) for f in pad]) 

	length = len (pad)
	if length == 1:   self.pad = pad * 4
	elif length == 2: self.pad = pad * 2
	elif length == 3: self.pad = pad + (pad[1],)
	elif length == 4: self.pad = pad
	else:
	    raise Exception, "A tuple of length 1-4 must be supplied"

    def getOffset (self, cellsize):
	"Calculates the offset to position the feature within a grid cell"

	offs = Vec3 (0, 0, 0)
	bb = self.object.getBBox ()
        do = bb[1] - bb[0]
        dc = Vec3 (cellsize[0], cellsize[1], 0)
    
	if 'l' in self.justification:
	    offs.x = -bb[0].x + self.getLPad()
	elif 'r' in self.justification:
	    offs.x = cellsize[0] - bb[1].x - self.getRPad()
	else:
            offs.x = 0.5*(dc.x-do.x) - bb[0].x

	if 'b' in self.justification:
	    offs.y = -bb[0].y + self.getBPad()
	elif 't' in self.justification:
	    offs.y = cellsize[1] - bb[1].y - self.getTPad()
	else:
            offs.y = 0.5*(dc.y-do.y) - bb[0].y
	    
	return offs

        
    def genFill (self, cellbase, cellsize):
	"Returns a list of triangles that fill a cell around this object"
	
	def MakeXYRect (t, l, b, r):
	    # don't make degenerates
	    if t == b or l == r: return []
	    
            # vertices
	    vtl = Vec3 (l, t, 0)
	    vtr = Vec3 (r, t, 0)
	    vbl = Vec3 (l, b, 0)
	    vbr = Vec3 (r, b, 0)

            # texture verts
            ttl = (l, t)
            ttr = (r, t)
            tbl = (l, b)
            tbr = (r, b)
    
	    tri0 = (vtr, vtl, vbl)
            tex0 = (ttr, ttl, tbl)
	    tri1 = (vtr, vbl, vbr)
            tex1 = (ttr, tbl, tbr)

	    return ((tri0, tex0), (tri1, tex1))
	    
	    
	filltris = []
	    
	omin,omax = self.object.getBBox ()
        cmin = cellbase
        cmax = Vec3 (cmin.x+cellsize[0], cmin.y+cellsize[1], cmin.z)
    
	# bottom
	filltris += MakeXYRect (omin.y, cmin.x, 
				cmin.y, cmax.x)

	# top
	filltris += MakeXYRect (cmax.y, cmin.x,
				omax.y, cmax.x)

	# left
	filltris += MakeXYRect (omax.y, cmin.x,
				omin.y, omin.x)
	
	# right
	filltris += MakeXYRect (omax.y, omax.x,
				omin.y, cmax.x)

	return filltris


        
def New (name, object, just='c', pad=(0,) ):
    "Returns a newly constructed Feature object"
    return Feature (name, object, just, pad)

