#
# Core file of the Procedural City Detailer
#
# The function you want to call is processScene()
#

import Registry, PRand, Scene, SceneTree, Util
import random, os, os.path, sys

# set this to True to print debugging stuff
verbose = False


def processScene (inscene, reg_file, building_callback=None):
    """Apply PCD to inscene using the registry filled by the python file
    reg_file. If building_callback is supplied, it is called after each
    building has been detailed."""
    
    registry = _createRegistry (reg_file)
    
    if verbose:
        print '---------------------------------------------'
        for i in registry.walkDir ('/'):
            print '/'+i
        print '---------------------------------------------'
        
    outscene = Scene.New ()

    style = lambda: registry.chooseObject('/styles').getObject()

    tree = SceneTree.Generate (inscene)
    
    for b in tree:
        prand = PRand.New ()
        outscene.addObject (style().fillBuilding (b, registry, prand))
        
        if building_callback != None:
            building_callback ()

    return outscene



def _createRegistry (reg_file):
    "Returns a registry filled by the python file reg_file"
    registry = Registry.New ()

    if verbose:
        print 'Loading registry from:', reg_file

    if not os.path.exists (reg_file):
        raise Exception, "Unable to load dataset, file does not exist: " \
                         + reg_file

    path,name = os.path.split (reg_file)
    if path not in sys.path:
        sys.path.append (path)
    if name[-3:] == '.py':
        name = name[:-3]
        
    Register = Util.Module.ImportName (name, 'Register')
    assert Register != None
    Register (registry)
    
    return registry
    

