from PCD.Util import XMLParse
from PCD import Material
import os, os.path


class ParseError(Exception):
    def __init__ (self, val):
        self.value = val
    def __str__ (self):
        return repr (self.val)
        
        
class MatDefParser (object):

    def __init__ (self, filename, texturedir=''):
        
        self.texturedir = texturedir
        self.materials = []
        
        tree = XMLParse.XMLFileToTree (filename)

        try:
            self._parseMaterials (tree)
        except ParseError, e:
            print >>sys.stderr, 'ERROR: Parse error in MATDEF file '+filename
            raise

        del self.texturedir


    ## parse functions
    
    def _parseMaterials (self, tree):
        assert tree[0] == 'materials'

        for material in tree[3]:
            self._parseMaterial (material)

    def _parseMaterial (self, tree):
        assert tree[0] == 'material'
        m = Material.Material (tree[1]['name'])
        
        for sub in tree[3]:
            if sub[0] == 'texchannel':
                chan = self._parseTexchannel (sub)
                m.setTexture (chan[0], chan[1])

            else:
                raise ParseError, "Unknown material subblock: "+sub[0]

        self.materials.append (m)

    def _parseTexchannel (self, tree):
        assert tree[0] == 'texchannel'
        num = int (tree[1]['num'])
        assert len (tree[3]) == 1
        tex = self._parseTexture (tree[3][0])
        return (num,tex)

    def _parseTexture (self, tree):
        assert tree[0] == 'texture'
        src = tree[1]['src']
        src = os.path.join (self.texturedir, src)
        src = os.path.abspath (src)
        
        return Material.Texture (src)

        


def RegisterDir (dirname, registry, texturedir=''):
    
    files = os.listdir (dirname)
    files = filter (lambda s: s[-7:] == '.matdef', files)
    files = [os.path.join (dirname, f) for f in files]
    
    for matdef in files:

        loader = MatDefParser (matdef, texturedir)

        for mat in loader.materials:
            registry.registerObject (mat, mat.name)

