import os, os.path


class Texture(object):

    def __init__ (self, filename):
        self.filename = filename
        self.name = os.path.basename (filename)

    def __hash__ (self):
        return hash (self.filename)

    def __eq__ (self, other):
        if not isinstance (other, Texture):
            return False
        return self.filename == other.filename

    def __ne__ (self, other):
        return not self.__eq__ (other)


    
class Material(object):

    NUMTEXTURES = 8

    def __init__ (self, name):
        self.name = name        
        self.textures = [None] * Material.NUMTEXTURES

    def __repr__ (self):
        return '<Material "%s">' % self.name

    def __str__ (self):
        return "Material '%s'" % self.name

    def __hash__ (self):
        return hash (tuple (self.textures))
    
    def __eq__ (self, other):
        if not isinstance (other, Material):
            return False
        for i in range (len (self.textures)):
            if self.textures[i] != other.textures[i]:
                return False
        return True

    def __ne__ (self, other):
        return not self.__eq__ (other)
    
    def setTexture (self, channel, texture):
        assert isinstance (texture, Texture)
        assert channel>=0 and channel<Material.NUMTEXTURES
        self.textures[channel] = texture

    def getTextures (self):
        return self.textures

    def getSaveName (self):
        stdpref = '/materials/'
        if self.name.startswith (stdpref):
            return self.name[len(stdpref):]
        else:
            return self.name
    

