"""A 4x4 Matrix class with some useful operators & functions

Operators:
 m1*m2, m1*=m2 	Matrix multiply
 v*m		Premultiplies vector by matrix
 m[n]		Returns the n'th row as a list

Alex Mole, 2003
"""

import math
import Vector3D


class Mat4:
    #irritating kludge
    _isMatrix = 1
    
    def __init__ (self, other=None):

	# initialise to identity
	self.setIdentity ()
		   
	# copy a 4x4 list if one has been passed
	if other != None:
	    if isMatrix (other):
                self.assign (other)
			
	    if len (other) != 4:
		raise TypeError, "Need 4x4 list to initialise matrix"
		
	    for i in range(4):
		if len (other[i]) != 4:
		    raise TypeError, "Need 4x4 list to initialise matrix"
		
		for j in range(4):
		    x = float (other[i][j])  # make sure we can cast to float
		    self.data[i][j] = x
		    
    
    def __repr__ (self):
	return "Matrix(%s)" % str (self.data)

    def __str__ (self):
	r0 = self.data[0]
	r1 = self.data[1]
	r2 = self.data[2]
	r3 = self.data[3]
	return ('[ %f %f %f %f]\n[%f %f %f %f]\n[%f %f %f %f]\n[%f %f %f %f]' %
		(r0[0], r0[1], r0[2], r0[3], r1[0], r1[1], r1[2], r1[3],
		 r2[0], r2[1], r2[2], r2[3], r3[0], r3[1], r3[2], r3[3]) )

    def __eq__ (self, o):
        for r in range (4):
            for c in range (4):
                if self.data[r][c] != o.data[r][c]:
                    return False
        return True

    # sequence-access
    def __len__ (self):
	return len (self.data)

    def __getitem__ (self, key):
	return self.data[key]

    def __setitem__ (self, key, value):
	self.data[key] = value

    def __iter__ (self):
	return self.data.__iter__()

    # (i,j) notation
    def __call__ (self, row, col):
        return self.data[row][col]
    
    # matrix-matrix or matrix-scalar multiply
    def __mul__ (self, rhs):
	if isMatrix (rhs):
	    a = self	# just to make the code easier to read
	    b = rhs
	    r = Mat4 ()
    
	    r[0][0] = a[0][0]*b[0][0] + a[0][1]*b[1][0] + 	\
					    a[0][2]*b[2][0] + a[0][3]*b[3][0]
	    r[0][1] = a[0][0]*b[0][1] + a[0][1]*b[1][1] + 	\
					    a[0][2]*b[2][1] + a[0][3]*b[3][1]
	    r[0][2] = a[0][0]*b[0][2] + a[0][1]*b[1][2] + 	\
					    a[0][2]*b[2][2] + a[0][3]*b[3][2]
	    r[0][3] = a[0][0]*b[0][3] + a[0][1]*b[1][3] + 	\
					    a[0][2]*b[2][3] + a[0][3]*b[3][3]

	    r[1][0] = a[1][0]*b[0][0] + a[1][1]*b[1][0] + 	\
					    a[1][2]*b[2][0] + a[1][3]*b[3][0]
	    r[1][1] = a[1][0]*b[0][1] + a[1][1]*b[1][1] + 	\
					    a[1][2]*b[2][1] + a[1][3]*b[3][1]
	    r[1][2] = a[1][0]*b[0][2] + a[1][1]*b[1][2] + 	\
					    a[1][2]*b[2][2] + a[1][3]*b[3][2]
	    r[1][3] = a[1][0]*b[0][3] + a[1][1]*b[1][3] + 	\
					    a[1][2]*b[2][3] + a[1][3]*b[3][3]

	    r[2][0] = a[2][0]*b[0][0] + a[2][1]*b[1][0] + 	\
					    a[2][2]*b[2][0] + a[2][3]*b[3][0]
	    r[2][1] = a[2][0]*b[0][1] + a[2][1]*b[1][1] + 	\
					    a[2][2]*b[2][1] + a[2][3]*b[3][1]
	    r[2][2] = a[2][0]*b[0][2] + a[2][1]*b[1][2] + 	\
					    a[2][2]*b[2][2] + a[2][3]*b[3][2]
	    r[2][3] = a[2][0]*b[0][3] + a[2][1]*b[1][3] + 	\
					    a[2][2]*b[2][3] + a[2][3]*b[3][3]

	    r[3][0] = a[3][0]*b[0][0] + a[3][1]*b[1][0] + 	\
					    a[3][2]*b[2][0] + a[3][3]*b[3][0]
	    r[3][1] = a[3][0]*b[0][1] + a[3][1]*b[1][1] + 	\
					    a[3][2]*b[2][1] + a[3][3]*b[3][1]
	    r[3][2] = a[3][0]*b[0][2] + a[3][1]*b[1][2] + 	\
					    a[3][2]*b[2][2] + a[3][3]*b[3][2]
	    r[3][3] = a[3][0]*b[0][3] + a[3][1]*b[1][3] + 	\
					    a[3][2]*b[2][3] + a[3][3]*b[3][3]

	    return r
					    
	else:
	    # attempt matrix-scalar
	    s = float (rhs)
	    r = Mat4 ()
	    for i in range(4):
		for j in range(4):
		    r[i][j] = self.data[i][j] * s
	    return r
	    
    def __imul__ (self, rhs):
	self = self * rhs
	return self
	    
    # vector-matrix premult
    def __rmul__ (self, v):
	if not Vector3D.isVector (v):
	    raise TypeError, "Only a vector can be premultiplied by a matrix"
	
	m = self.data		# shorthand
	
	res = Vector3D.Vec3()
	res.x = v.x*m[0][0] + v.y*m[1][0] + v.z*m[2][0] + m[3][0]
	res.y = v.x*m[0][1] + v.y*m[1][1] + v.z*m[2][1] + m[3][1]
	res.z = v.x*m[0][2] + v.y*m[1][2] + v.z*m[2][2] + m[3][2]
    
	return res

    def assign (self, other):
        "Assigns the data of another Mat4 to self"
        assert isMatrix (other)
        for i in range(4):
            for j in range(4):
                self.data[i][j] = other.data[i][j]

    def assignString (self, str):
	"Assigns a string containing 16 whitespace-delimited values to matrix"
	l = str.split()
	assert len(l) == 16
	
	for i in range(4):
	    for j in range(4):
		self.data[i][j] = float (l[i*4 + j])

    def setIdentity (self):
	"Sets this matrix to the identity"
	self.data = [ [1., 0., 0., 0.], 
		      [0., 1., 0., 0.],
		      [0., 0., 1., 0.], 
		      [0., 0., 0., 1.] ]
	
	
# type check
def isMatrix (m):
   return hasattr (m, '_isMatrix') 
    

# matrix construction routines
def Translation (v):
    "Returns a 4x4 matrix representing the translation specified"
    m = Mat4 ()
    m[3][0] = v[0]
    m[3][1] = v[1]
    m[3][2] = v[2]
    return m

def EulerRotation (r):
    "Returns a 4x4 matrix representing Blender style Euler rotations"
    cp = math.cos (r[0])
    ch = math.cos (r[1])
    cr = math.cos (r[2])
    sp = math.sin (r[0])
    sh = math.sin (r[1])
    sr = math.sin (r[2])
    cpcr = cp * cr 
    cpsr = cp * sr 
    spcr = sp * cr 
    spsr = sp * sr

    m = Mat4 ()
    m[0][0] = ch * cr
    m[1][0] = (sh * spcr) - cpsr
    m[2][0] = (sh * cpcr) + spsr
    m[0][1] = ch * sr
    m[1][1] = (sh * spsr) + cpcr
    m[2][1] = (sh * cpsr) - spcr
    m[0][2] = -sh
    m[1][2] = ch * sp
    m[2][2] = ch * cp

    return m


def Scaling (*s):
    """Returns the 4x4 matrix for the scale values supplied.
    Accepts 1 or 3 floats or a list of 3 floats."""
    m = Mat4 ()
    if len (s) == 1:
        if hasattr (s, len):
            if len (s) != 3:
                raise TypeError, 'Scaling() accepts s, xyz or [xyz]'

            # args like [xyx]
            for i in range(3):
                m[i][i] = float (s[0][i])
                
        else:
            # args like s
            m[0][0] = m[1][1] = m[2][2] = float(s[0])
            
    elif len (s) == 3:
        # args like xyz
        for i in range(3):
            m[i][i] = float (s[i])
            
    else:
        raise TypeError, 'Scaling() accepts s, xyz or [xyz]'
        
    return m




# matrix transformation routines
# these all PREMULTIPLY
def ApplyTranslation (m, v):
    t = Translation (v)
    m = t * m
    return m

    

# "removes" the scaling factors from a rotation matrix
def UnscaleMatrix (m):
    """Extracts the scaling factors from the matrix, normalises the 
    rotation kernel and returns the scaling factors as a 3-tuple"""
    X = Vector3D.Vec3 (m[0][0], m[1][0], m[2][0])
    Y = Vector3D.Vec3 (m[0][1], m[1][1], m[2][1])
    Z = Vector3D.Vec3 (m[0][2], m[1][2], m[2][2])
    lx = X.length ()
    ly = Y.length ()
    lz = Z.length ()
    X = X / lx
    Y = Y / ly
    Z = Z / lz
    m[0][0],m[1][0],m[2][0] = X.x,X.y,X.z
    m[0][1],m[1][1],m[2][1] = Y.x,Y.y,Y.z
    m[0][2],m[1][2],m[2][2] = Z.x,Z.y,Z.z
    return lx,ly,lz


    
