"""3D maths utility functions

Alex Mole, 2003
"""

from PCD.Maths.Vector3D import *
import math


def FaceNormal (verts):
    """Returns the normal of the face defined by the vertices in the list.
    
    The verts must be wound anticlockwise. Only the first three are used."""
    
    e0 = verts[1] - verts[0]
    e1 = verts[2] - verts[0]
    normal = e0.cross (e1)
    normal.normalise ()

    return normal

    

def TriArea (a, b, c):
    """Returns the area of a triangle.

    Arguments are position vectors describing the triangle.
    """
    # type checking: we need vectors
    if not (isVector(a) and isVector(b) and isVector(c)):
	raise TypeError, "tri_area: Needs 3 vectors as arguments"

    AB = b - a
    AC = c - a

    cross = AB.cross (AC)

    return 0.5 * math.fabs (cross.length())

    

def TriAngle (a, b, c, d):
    """Returns the angle between two triangles that share an edge.

    All arguments are position vectors, and AB is the shared edge.
    Triangles should be wound so that ABC and ADB are anticlockwise.
    """
    # measure the angle between the normals and subtract from pi to get result
    
    # type checking: we need vectors
    if not (isVector(a) and isVector(b) and isVector(c) and isVector(d)):
	raise TypeError, "tri_angle: Needs 4 vectors as arguments"

    # find the normals
    AB = b - a
    AC = c - a
    AD = d - a

    n0 = AB.cross (AC)
    n1 = AD.cross (AB)

    n0.normalise()
    n1.normalise()

    # get the dot product (= cos(angle))
    dot = n0.dot (n1)

    # work out the angle between the triangles
    return math.pi - math.acos (dot)



def MakeXForm (offset, yrot=0):
    "Returns a function object to filter vectors, rotating then translating"
    c = math.cos
    s = math.sin
    rots = ( c(yrot), s(yrot), -s(yrot), c(yrot) )
    return lambda v,o=offset,r=rots:	\
	    Vec3 (  o[0] + r[0]*v.x + r[1]*v.z,
		    o[1] + v.y,
		    o[2] + r[2]*v.x + r[3]*v.z )
    
	



