from PCD import Material, Object
from PCD.Maths import Vector3D


def New (name='Meta'):
    "Creates and returns a new MetaObject"
    return MetaObject (name)


def FromScene (scene, name='Meta'):
    "Creates a new MetaObject and fills it with the contents of the Scene"
    m = MetaObject (name)
    for o in scene.getObjects():
        m.addObjectData (o)
    return m


class MetaObject:
    "Defines an object with more than one texture"

    def __init__ (self, name="Meta"):
        self.objects = {}
        self.name = name
	self._bb = (Vector3D.Vec3(0,0,0), Vector3D.Vec3(0,0,0))
        self._bb_valid = False

    def __repr__ (self):
	return 'MetaObject(%s, %s)' % (self.name, 
                                       repr (self.objects.values()))
    
        
    def clone (self):
        "Returns a copy of this MetaObject"
        mo = MetaObject (self.name)
        for o in self.getObjects():
            mo.objects[o.material] = o.clone ()

        if self._bb_valid:
            o._bb = self._bb
            o._bb_valid = True
            
        return mo

    def setTransform (self, t):
        for o in self.getObjects():
            o.setTransform (t)
        self._bb_valid = False
        
    def applyTransformFunc (self, transform):
        for o in self.getObjects():
            o.applyTransformFunc (transform)
        self._bb_valid = False

    def translate (self, t):
        "Translates the object by vector t"
        self.applyTransformFunc (lambda v,s=t: v+s)    
        
    def getObjects (self):
        return self.objects.values ()
    
    def addObjectData (self,other):
	"Adds the data from another object to this one"
        assert other is not None
        if isinstance (other, MetaObject):
            for o in other.getObjects():
                self.addObjectData (o)
        else:
            try:    
                self.objects[other.material].addObjectData (other)
            except KeyError:
                self.objects[other.material] = other.clone ()
        self._bb_valid = False


    def calcBBox (self):
        mins = [o.getBBox()[0] for o in self.getObjects()]
        maxs = [o.getBBox()[1] for o in self.getObjects()]
        min = Vector3D.GetMin (mins)
        max = Vector3D.GetMax (maxs)
        self._bb = (min,max)
        self._bb_valid = True
    
    def getBBox (self):
	"Returns the bounding box as tuple: (min, max)"
        if not self._bb_valid:
            self.calcBBox ()
	return self._bb

    def getHeight (self):
	"Returns height of the object (y-axis)"
        bb = self.getBBox ()            
	return bb[1].y - bb[0].y
	
    def getWidth (self):
	"Returns width of the object (x-axis)"
        bb = self.getBBox ()            
	return bb[1].x - bb[0].x
	


