from PCD import MetaObject, Object, Material


def New ():
    "Creates & returns a new Scene object"
    return Scene ()



class Scene(object):
    
    def __init__ (self):
        self.objects = {}
        self.materials = {}
        self.textures = {}


    def getTextures (self):
        "Returns the list of Textures"
        return self.textures.values ()

    def getMaterials (self):
        "Returns the list of Materials"
        return self.materials.values ()

    def getObjects (self):
        "Returns the list of Objects"
        return self.objects.values ()


    def getTexture (self, name):
        "Returns a Texture for the filename, or None if it doesn't exist"
        try:
            return self.textures[name]
        except KeyError:
            return None

    def newTexture (self, filename, name=None):
        "Returns a new texture with the supplied information"
        tex = Material.Texture (filename)
        if name: tex.name = name
        self.textures[tex.name] = tex
        return tex
        
    def addTexture (self,texture):
        "Adds the texture. Ignored if it is already in the scene"
        if texture == None: return

        owned = False
        for otex in self.textures.values():
            if otex == texture:
                texture.name = otex.name
                owned = True
                break

        if not owned:
            texture.name = self.__getSafeName (texture.name, self.textures)
            self.textures[texture.name] = texture


    def getMaterial (self, name):
        "Returns the named Material, or None if it does not exist"
        try:
            return self.materials[name]
        except KeyError:
            return None

    def newMaterial (self, name):
        """Creates a new material, adds it to the scene and returns it

        Note that the name may change in order to resolve clashes"""
        name = self.__getSafeName (name, self.materials)
        mat = Material.Material (name)
        self.materials[name] = mat
        return mat

    def addMaterial (self, material):
        "Adds the material and textures. Ignored if it is already in the scene"
        if material == None: return
        owned = False
        for item in self.materials.items ():
            # we may need to update the name within this material to match
            # the dictionary key
            if item[1] == material:
                material.name = item[1].name
                owned = True
                break

        if not owned:
            material.name = self.__getSafeName (material.name, self.materials)
            self.materials[material.name] = material

        for tex in material.getTextures ():
            self.addTexture (tex)


    def getObject (self, name):
        "Returns the named Object, or None if it does not exist"
        try:
            return self.objects[name]
        except KeyError:
            return None
        
    def newObject (self, name):
        """Creates a new Object, adds it to the scene and returns it

        Note that the name may change in order to resolve clashes"""
        name = self.__getSafeName (name, self.objects)
        obj = Object.New (name)
        self.objects[name] = obj
        return obj

    def addObject (self, obj):
	"Adds an Object or a MetaObject to the scene (renames it if necessary)"
        if isinstance (obj, MetaObject.MetaObject):
            for o in obj.getObjects ():
                self.addObject (o)
        else:
	    obj.name = self.__getSafeName (obj.name, self.objects)
	    self.objects[obj.name] = obj
            self.addMaterial (obj.material)

        
    def __getSafeName (self, name, d):
	suffixnum = 0
	orgname = name	# save original name
	
	while True:
            # have we found an unused name?
	    if name not in d:
                break
                
	    # if the name already exists, try a different suffix
            suffixnum += 1
            name = '%s.%03d' % (orgname, suffixnum)
		
	return name






