from PCD.Maths import Vector3D, Matrix


def New (pos=None, euler=None, scale=None):
    return Transform (pos, euler, scale)


class Transform:
    """A general transform class, representing euler rot, scale and 
    translation. Initialise with up to three 3-tuples"""
    

    def __init__ (self, pos=None, euler=None, scale=None):
        self.clear ()
        
        if pos: 
            self.pos = Vector3D.Vec3 (pos)
        if euler:
            self.rot = Vector3D.Vec3 (euler)
        if scale:
            self.sca = Vector3D.Vec3 (scale)


    def assign (self, o):
        "Copies the contents of another Transform into self"
        self.pos.assign (o.pos)
        self.rot.assign (o.rot)
        self.sca.assign (o.sca)


    def clear (self):
        "Sets the transform to the identity"
        self.pos = Vector3D.Vec3 (0,0,0)
        self.rot = Vector3D.Vec3 (0,0,0)
        self.sca = Vector3D.Vec3 (1,1,1)


    def setPosition (self, p):
        "Sets the translation part of the transform"
        self.pos = Vector3D.MakeVector (p)

    def setRotation (self, r):
        "Sets the rotation part of the transformation"
        self.rot = Vector3D.MakeVector (r)
        
    def setScale (self, s):
        "Sets the scaling part of the transformation"
        self.sca = Vector3D.MakeVector (s)
        

    def getMatrix (self):
        """Returns the transform in matrix form.
        Order: Scale, Rotate, Translate"""
        s = Matrix.Scaling (self.sca)
        r = Matrix.EulerRotation (self.rot)
        t = Matrix.Translation (self.pos)
        return t * r * s



