"""Hashed list class

Acts like a list, but membership tests are O(1).
Only works for hashable items.

Based on recipe by Alex Martelli

Alex Mole, 2003
"""


__all__ = ['HashedList', 'New']


class HashedList (list):

    def __init__ (self):
	list.__init__ (self)
	self._dict_ok = 0
	self._dict = {}

    def rebuildDict (self):
        if not self._dict_ok:
            self._dict = dict(zip(self,xrange(len(self))))
            self._dict_ok = 1
        
    def __contains__ (self, item):
        self.rebuildDict ()
	return item in self._dict

    # override the built-in append
    def append (self, item):
	self._dict[item] = len (self)
	list.append (self, item)

    def index (self, item):
        try:
            self.rebuildDict ()
            return self._dict[item]
        except KeyError:
            raise ValueError, 'HashedList.index(x): x not in list'



def New ():
    return HashedList ()
	
	
#	
# shortcut to wrap up the list methods and make them all safe
#
	
def _wrapMethod (methname):
    _method = getattr (list, methname)
    
    def wrapper (self, *args):
	# reset dictionary OK flag then delegate to normal method
	self._dict_ok = 0
	return _method (self, *args)

    setattr (HashedList, methname, wrapper)

# wrap __*__ methods
for meth in 'setitem delitem setslice delslice iadd'.split():
    _wrapMethod ('__%s__' % meth)

# wrap other methods
for meth in 'append insert pop remove extend'.split():
    _wrapMethod (meth)

del _wrapMethod


