"""
XML Parsing utilities

Alex Mole, 2003
"""

import sys, xml.parsers.expat


def XMLFileToTree (filename):
    """Loads an xml file and stores it as a tree [recursive list]

    Format of each node is: [tag, attrs, cdata, children]
             tag & cdata are strings, attrs is a dict and children is a list
    
    Example Input:
        <head name='jeff'>
          <obj name='joe' size="3" />
          <bigobj name='karl'>
            <leaf>hello</leaf>
          </bigobj>
        </head>

    Output:
    ['head', {'name':'jeff'}, '', [ 
        ['obj', {'name:'joe','size':'3'}, '', [] ],
        ['bigobj', {'name':'karl'}, '', [ ['leaf', {}, 'hello', [] ] ] ] ] ]
    """

    f = file (filename, 'r')
    text = f.read ()
    f.close ()

    p = _XMLTreeParser ()
    return p.Parse (text)


    
class XMLParseError(Exception): pass


class _XMLTreeParser(object):
    
    def __init__ (self):
	self.parser = xml.parsers.expat.ParserCreate()
	self.parser.StartElementHandler = self._starttag
	self.parser.EndElementHandler = self._endtag
	self.parser.CharacterDataHandler = self._chardata

	# the text data for the current element
	self.currdata = ''


    def Parse (self, text):
        self.currel = [ ['DUMMY', {}, '', []] ]
        self.parser.Parse (text, 1)
        root = self.currel.pop ()
        return root[3][0]        


    # these are parser callbacks
    def _starttag (self, tag, attrs):
        self.currel[-1][2] += self.currdata
        self.currdata = ''
	self.currel.append ([ tag,attrs,'',[] ])
    
    def _endtag (self, tag):
	node = self.currel.pop ()
        node[2] += self.currdata
        self.currdata = ''
        self.currel[-1][3].append (node)
	    
    def _chardata (self, data):
	self.currdata += data



if __name__ == '__main__':
    if len (sys.argv) > 1:
        print XMLFileToTree (sys.argv[1])

