from PCD.Maths.Vector3D import *
import unittest


class Vec3TestCase (unittest.TestCase):

    def setUp (self):
        self.zero = Vec3 (0,0,0)
        self.vb = Vec3 (1,2,3)
        self.vc = Vec3 (7,7,2)
        self.vd = Vec3 (7,7,3)
        self.ve = Vec3 (1,2,3)
    
    def testDefault (self):
        assert Vec3() == Vec3(0,0,0)
    
    def testEquality (self):
        assert self.vb == self.vb
        assert not (self.vb == self.vc)
        assert not (self.vc == self.vd)
        assert self.vb == self.ve

    def testInequality (self):
        assert self.zero != self.vb
        assert self.vc != self.vd
        assert not (self.vb != self.ve)
        
    def testAddition (self):
        assert self.zero + self.vb == self.vb
        assert self.vb + self.zero == self.vb
        assert self.vb + self.vc == Vec3 (8,9,5)

    def testIAddition (self):
        a = Vec3(1,1,1)
        a += self.zero
        assert a == Vec3(1,1,1)
        b = Vec3(4,5,6)
        b += Vec3(2,4,6)
        assert b == Vec3(6,9,12)

    def testSubtraction (self):
        assert self.vb - self.zero == self.vb
        assert Vec3(1,2,3) - Vec3(2,2,2) == Vec3(-1,0,1)

    def testISubtraction (self):
        a = Vec3(4,5,6)
        a -= Vec3 (2,2,2)
        assert a == Vec3(2,3,4)
        b = Vec3(9,8,7)
        b -= Vec3(0,0,0)
        assert b == Vec3(9,8,7)

    def testScalarMultiplication (self):
        assert 3 * self.zero == self.zero
        assert self.zero * 5 == self.zero
        assert 2 * Vec3(3,4,5) == Vec3(6,8,10)
        assert Vec3(6,5,2) * 6 == Vec3(36,30,12)

    def testIScalarMultiplication (self):
        a = Vec3(3,6,2)
        a *= 4
        assert a == Vec3(12,24,8)

    def testDivision (self):
        assert self.zero / 3 == self.zero
        assert Vec3(12,24,4) / 4 == Vec3(3,6,1)

    def testIDivision (self):
        a = Vec3(25,35,10)
        a /= 5
        assert a == Vec3(5,7,2)

    def testDot (self):
        assert self.zero.dot (self.vb) == 0
        assert self.vb.dot (self.vc) == 27

    def testCross (self):
        assert self.zero.cross (self.vb) == self.zero
        assert self.vb.cross (self.zero) == self.zero
        a = Vec3 (1,2,3)
        b = Vec3 (5,2,1)
        assert a.cross (b) == Vec3 (-4, 14, -8)
        assert b.cross (a) == Vec3 (4, -14, 8)

    def testLength (self):
        assert self.zero.length () == 0
        assert Vec3(3,4,12).length () == 13
    
    def testLengthSq (self):
        assert self.zero.length_sq () == 0
        assert Vec3 (4,6,2).length_sq () == 56
    



def suite ():
    return unittest.makeSuite (Vec3TestCase)
    

if __name__ == "__main__":
    unittest.main ()

