from PCD.SceneTree import *
import unittest


class SceneTreeTestCase (unittest.TestCase):

    def setUp (self):
        self.triA = [Vec3(0,0,0), Vec3(1,0,0), Vec3(1,1,0)]
        self.edgeA = [Vec3(1,0,0), Vec3(0,1,0), Vec3(-1,-1,0)]
        self.triB = [Vec3(0,0,0), Vec3(0,-1,0), Vec3(1,0,0)]
        self.triC = [Vec3(0,0,0), Vec3(1,1,0), Vec3(1,1,1)]
        self.tris = [[Vec3(1,0,4), Vec3(2,1,0), Vec3(1,1,0)],
                     [Vec3(1,0,0), Vec3(2,1,0), Vec3(1,1,0)],
                     [Vec3(0,0,0), Vec3(1,1,0), Vec3(0,1,0)],
                     [Vec3(0,0,0), Vec3(-1,0,0), Vec3(0,-1,0)]]

    def testNormalCalc (self):
        f = Face (self.triA)
        self.assertEqual (f.normal, Vec3(0,0,1))
        
    def testAddTri (self):
        f = Face (self.triA)
        self.failUnless (f.tryAddTri (self.triB), 
                                        'failed to add valid triangle')
        self.failIf (f.tryAddTri (self.triC), 'added invalid triangle')

        f.addAllTrisInList (self.tris)
        self.failUnless (len(self.tris) == 1, 
                            'trilist incorrectly updated by addAllTrisInList')
        self.failUnlessEqual (self.tris[0], 
                    [Vec3(1,0,4), Vec3(2,1,0), Vec3(1,1,0)], 
                    'triangle list incorrectly updated after addAllTrisInList')

        res = [ Vec3(0, 1, 0), Vec3(0, 0, 0), Vec3(-1, 0, 0), Vec3(0, -1, 0), 
                                    Vec3(1, 0, 0), Vec3(2, 1, 0), Vec3(1, 1, 0)]
        self.assertEqual (f.verts, res)

    def testGetEdges (self):
        f = Face (self.triA)
        self.assertEqual (f.getEdges(), self.edgeA)



def suite ():
    return unittest.makeSuite (SceneTreeTestCase)
    

if __name__ == "__main__":
    unittest.main ()

    
