from PCD import MetaObject, Object
from PCD.Architecture import FeatGen
from PCD.Maths.Vector3D import Vec3


class RecessedDFFunc (FeatGen.Func):
    "Simple recessed door frame"
    
    def __init__ (self):
        self._regpath = '/features/door/frames'
        self._regname = 'simple-recessed'
        self._regsockets = [ ('object','/features/door/objects'),
                             ('mat', '/materials/basic') ]


    def generateContent (self, size, registry, prand, level):

        w,h = size
        d = prand.val ('frame-depth', -0.03, -0.50)

        ov = [ Vec3(0,0,0), Vec3(0,h,0), Vec3(w,h,0), Vec3(w,0,0) ]
        dv = [ v + Vec3(0,0,d) for v in ov ]
        ot = [ (0,0), (0,1), (1,1), (1,0) ]
        dt = [ (0.5,0.5) ] * 4

        frame = Object.New ()
        robj = self.chooseSocketValue ('mat', registry, prand)
        frame.material = robj.getObject ()
        self._genQuadStrip (frame, ov, dv, ot, dt)
        
        obj = self.socketGenContent ('object', size, registry,
                                                    prand, level, (0,0,d))
        
        all = MetaObject.New ()
        all.addObjectData (frame)
        all.addObjectData (obj)
        
        return all



class Box3DFFunc (FeatGen.Func):
    "Door frame with a 'box' around 3 sides"
    
    def __init__ (self):
        self._regpath = '/features/door/frames'
        self._regname = 'box3'
        self._regsockets = [ ('object','/features/door/objects'),
                             ('mat', '/materials/basic') ]


    def generateContent (self, size, registry, prand, level):

        w,h = size
        od = prand.val ('bframe-odepth', 0.03, 0.25)
        id = prand.val ('bframe-idepth', -0.03, -0.50)
        t = prand.val ('bframe-thickness', 0.10, 0.20)

        # generate some verts...
        TexCoords = lambda f: [(f,f), (f,1-f), (1-f,1-f), (1-f,f)]
        
        obv = [ Vec3(0,0,0), Vec3(0,h,0), Vec3(w,h,0), Vec3(w,0,0) ]
        obt = TexCoords (0)
        ofv = [ v + Vec3(0,0,od) for v in obv]
        oft = TexCoords (1.0/6.0)
        ifv = [ Vec3(t,0,od), Vec3(t,h-t,od), Vec3(w-t,h-t,od), Vec3(w-t,0,od)]
        ift = TexCoords (2.0/6.0)
        ibv = [ v + Vec3(0,0,id) for v in ifv]
        ibt = TexCoords (0.5)

        frame = Object.New ()
        robj = self.chooseSocketValue ('mat', registry, prand)
        frame.material = robj.getObject ()
        self._genQuadStripN (frame, obv, ofv, obt, oft)
        self._genQuadStripN (frame, ofv, ifv, oft, ift)
        self._genQuadStrip (frame, ifv, ibv, ift, ibt)
        
        csize = (w-2*t, h-t)
        obj = self.socketGenContent ('object', csize, registry,
                                                prand, level, (t,0,od+id))
        
        all = MetaObject.New ()
        all.addObjectData (frame)
        all.addObjectData (obj)
        
        return all



def Register ():
    return [ RecessedDFFunc(), Box3DFFunc() ]


