from PCD import MetaObject, Object
from PCD.Architecture import FeatGen
from PCD.Maths.Vector3D import Vec3


class BasicWindowContainerFunc (FeatGen.Func):
    "Basic window container type- just passes straight through to pane"
    
    def __init__ (self):
        self._regpath = '/features/window/containers'
        self._regname = 'basic'
        self._regsockets = [ ('pane','/features/window/panes') ]


    def generateContent (self, size, registry, prand, level):
        return self.socketGenContent ('pane', size, registry, prand, level)



class FourSectionsWinContFunc (FeatGen.Func):
    "Divides the window vertically and horizontally into a 2x2 grid"
    
    def __init__ (self):
        self._regpath = '/features/window/containers'
        self._regname = 'four-sections'
        self._regsockets = [ ('pane','/features/window/panes/nontrivial') ]


    def generateContent (self, size, registry, prand, level):

        w,h = size
        hw = w / 2.0
        hh = h / 2.0
        psize = (hw,hh)
        
        bl = self.socketGenContent ('pane', psize, registry, prand, level)
        br = self.socketGenContent ('pane', psize, registry, 
                                               prand, level, (hw,0,0))
        tl = self.socketGenContent ('pane', psize, registry, 
                                               prand, level, (0,hh,0))
        tr = self.socketGenContent ('pane', psize, registry, 
                                               prand, level, (hw,hh,0))
        
        all = MetaObject.New ()
        all.addObjectData (bl)
        all.addObjectData (br)
        all.addObjectData (tl)
        all.addObjectData (tr)

        return all
         


class ThreeSectionsWinContFunc (FeatGen.Func):
    "Divides the window horizontally, then one half vertically"
    
    def __init__ (self):
        self._regpath = '/features/window/containers'
        self._regname = 'three-sections'
        self._regsockets = [ ('pane','/features/window/panes/nontrivial') ]


    def generateContent (self, size, registry, prand, level):

        w,h = size
        
        all = MetaObject.New ()

        if h > w:
            dh = h * prand.val ('cont-hsplit', 0.6, 0.8)
            dw = w/2.0
            top = self.socketGenContent ('pane', (w,h-dh), registry, 
                                                        prand, level, (0,dh,0))
            bl = self.socketGenContent ('pane', (dw,dh), registry, prand, level)
            br = self.socketGenContent ('pane', (dw,dh), registry, 
                                                        prand, level, (dw,0,0))
        
            all.addObjectData (top)
            all.addObjectData (bl)
            all.addObjectData (br)

        else:
            dh = h/2.0
            dw = w * prand.val ('cont-vsplit', 0.5, 0.8)

            # bigger section on left or right
            if prand.num ('cont-side', 2) == 0:
                top = self.socketGenContent ('pane', (dw,dh), registry, 
                                                       prand, level, (0,dh,0)) 
                bot = self.socketGenContent ('pane', (dw,dh), registry, prand, 
                                                                        level)
                side = self.socketGenContent ('pane', (w-dw,h), registry, 
                                                        prand, level, (dw,0,0))
                
            else:
                top = self.socketGenContent ('pane', (dw,dh), registry, 
                                                    prand, level, (w-dw,dh,0)) 
                bot = self.socketGenContent ('pane', (dw,dh), registry, prand, 
                                                             level, (w-dw,0,0))
                side = self.socketGenContent ('pane', (w-dw,h), registry, 
                                                                prand, level)
        
            all.addObjectData (top)
            all.addObjectData (bot)
            all.addObjectData (side)

        return all
         




def Register ():
    return [ BasicWindowContainerFunc(), 
             FourSectionsWinContFunc(),
             ThreeSectionsWinContFunc() ]


