from PCD import MetaObject, Object
from PCD.Architecture import FeatGen
from PCD.Maths.Vector3D import Vec3


class RecessedWFFunc (FeatGen.Func):
    "Simple recessed window frame"
    
    def __init__ (self):
        self._regpath = '/features/window/frames'
        self._regname = 'simple-recessed'
        self._regsockets = [ ('container','/features/window/containers'),
                             ('mat', '/materials/basic') ]


    def generateContent (self, size, registry, prand, level):

        w,h = size
        d = prand.val ('frame-depth##'+str(level), -0.03, -0.50)

        ov = [ Vec3(0,0,0), Vec3(0,h,0), Vec3(w,h,0), Vec3(w,0,0) ]
        iv = [ v + Vec3(0,0,d) for v in ov ]
        ot = [ (0,0), (0,1), (1,1), (1,0) ]
        it = [ (0.5,0.5) ] * 4
        
        frame = Object.New ()
        robj = self.chooseSocketValue ('mat', registry, prand)
        frame.material = robj.getObject ()
        self._genQuadStrip (frame, ov, iv, ot, it)
        
        container = self.socketGenContent ('container', size, registry,
                                                    prand, level, (0,0,d))
        
        all = MetaObject.New ()
        all.addObjectData (frame)
        all.addObjectData (container)
        
        return all



class BoxWFFunc (FeatGen.Func):
    "Simple recessed window frame"
    
    def __init__ (self):
        self._regpath = '/features/window/frames'
        self._regname = 'box'
        self._regsockets = [ ('container','/features/window/containers'),
                             ('mat', '/materials/basic') ]


    def generateContent (self, size, registry, prand, level):

        w,h = size
        od = prand.val ('bframe-odepth##'+str(level), 0.03, 0.20)
        id = prand.val ('bframe-idepth##'+str(level), -0.03, -0.50)
        t = prand.val ('bframe-thickness##'+str(level), 0.08, 0.20)

        # generate some verts...
        TexCoords = lambda f: [(f,f), (f,1-f), (1-f,1-f), (1-f,f)]
        
        obv = [ Vec3(0,0,0), Vec3(0,h,0), Vec3(w,h,0), Vec3(w,0,0) ]
        obt = TexCoords (0)
        ofv = [ v + Vec3(0,0,od) for v in obv]
        oft = TexCoords (1.0/6.0)
        ifv = [ Vec3(t,t,od), Vec3(t,h-t,od), Vec3(w-t,h-t,od), Vec3(w-t,t,od)]
        ift = TexCoords (2.0/6.0)
        ibv = [ v + Vec3(0,0,id) for v in ifv]
        ibt = TexCoords (0.5)

        frame = Object.New ()
        robj = self.chooseSocketValue ('mat', registry, prand)
        frame.material = robj.getObject ()
        self._genQuadStrip (frame, obv, ofv, obt, oft)
        self._genQuadStrip (frame, ofv, ifv, oft, ift)
        self._genQuadStrip (frame, ifv, ibv, ift, ibt)
        
        csize = (w-2*t, h-2*t)
        container = self.socketGenContent ('container', csize, registry,
                                                prand, level, (t,t,od+id))
        
        all = MetaObject.New ()
        all.addObjectData (frame)
        all.addObjectData (container)
        
        return all



def Register ():
    return [ RecessedWFFunc(), BoxWFFunc() ]


