from PCD import MetaObject, Object
from PCD.Architecture import FeatGen
from PCD.Maths.Vector3D import Vec3


class BasicWindowPaneFunc (FeatGen.Func):
    "Basic window pane type- just passes straight through to glass"
    
    def __init__ (self):
        self._regpath = '/features/window/panes'
        self._regname = 'basic'
        self._regsockets = [ ('glass','/features/window/glass') ]


    def generateContent (self, size, registry, prand, level):
        return self.socketGenContent ('glass', size, registry, prand, level)



class ThinFramePaneFunc (FeatGen.Func):
    "Pane with a thin border around it"

    def __init__ (self):
        self._regpath = '/features/window/panes/nontrivial'
        self._regname = 'thin-frame'
        self._regsockets = [ ('glass','/features/window/glass'),
                             ('mat', '/materials/basic')            ]


    def generateContent (self, size, registry, prand, level):

        w,h = size
        t = prand.val ('paneframe-thickness', 0.02, 0.06)
        d = prand.val ('paneframe-depth', -0.01, -0.04)

        # generate some verts
        ov = [ Vec3(0,0,0), Vec3(0,h,0), Vec3(w,h,0), Vec3(w,0,0) ]
        iv = [ Vec3(t,t,0), Vec3(t,h-t,0), Vec3(w-t,h-t,0), Vec3(w-t,t,0) ]
        dv = [ v + Vec3(0,0,d) for v in iv]
        
        # ...and some texcoords
        T = lambda v: (v.x,v.y)
        ot = map (T, ov)
        it = map (T, iv)
        dt = map (T, dv)
        
        frame = Object.New ()

        robj = self.chooseSocketValue ('mat', registry, prand)
        frame.material = robj.getObject ()
        
        self._genQuadStrip (frame, ov, iv, ot, it)
        self._genQuadStrip (frame, iv, dv, it, dt)
    
        gsize = (w-2*t,h-2*t)
        glass = self.socketGenContent ('glass', gsize, registry, 
                                                prand, level, (t,t,d))
        
        all = MetaObject.New ()
        all.addObjectData (frame)
        all.addObjectData (glass)

        return all
        


class BevelFramePaneFunc (FeatGen.Func):
    "Pane with a thin beveled border around it"

    def __init__ (self):
        self._regpath = '/features/window/panes/nontrivial'
        self._regname = 'bevel-frame'
        self._regsockets = [ ('glass','/features/window/glass'),
                             ('mat', '/materials/basic')            ]


    def generateContent (self, size, registry, prand, level):

        w,h = size
        t = prand.val ('paneframe-thickness', 0.02, 0.06)
        d = prand.val ('paneframe-depth', -0.02, -0.06)

        # generate some verts
        ov = [ Vec3(0,0,0), Vec3(0,h,0), Vec3(w,h,0), Vec3(w,0,0) ]
        iv = [ Vec3(t,t,d), Vec3(t,h-t,d), Vec3(w-t,h-t,d), Vec3(w-t,t,d) ]
        
        # ...and some texcoords
        T = lambda v: (v.x,v.y)
        ot = map (T, ov)
        it = map (T, iv)
        
        frame = Object.New ()
        
        robj = self.chooseSocketValue ('mat', registry, prand)
        frame.material = robj.getObject ()
        
        self._genQuadStrip (frame, ov, iv, ot, it)
    
        gsize = (w-2*t,h-2*t)
        glass = self.socketGenContent ('glass', gsize, registry, 
                                                prand, level, (t,t,d))
        
        all = MetaObject.New ()
        all.addObjectData (frame)
        all.addObjectData (glass)

        return all
        


def Register ():
    return [ BasicWindowPaneFunc(), ThinFramePaneFunc(), BevelFramePaneFunc() ]


