from PCD import MetaObject, Object
from PCD.Architecture import Roof
from PCD.Maths.Vector3D import Vec3
import PCD.Maths.Util
import math, operator, random


class Simple (Roof.Func):
    "Totally flat roof"

    def __init__ (self):
        self._regpath = '/roofs'
        self._regname = 'flat-simple'
        self._regsockets = [ ('mat', '/materials/roof/flat'),
                             ('decor', '/objects/decoration/roof/flat') ]


    def __genDecor (self, face, registry, prand):
        
        i = random.randrange (len (face.verts))
        ni = (i+1) % len (face.verts)
        
        e = face.verts[ni] - face.verts[i]
        n = e.cross (face.normal)
        n.normalise()
        el = e.length ()

        obj = self.chooseSocketValue ('decor', registry, prand).getObject ()

        if el < obj.getWidth():
            # we can't add decor here, it's too small
            return Object()
        
        s = random.uniform (0.2, 0.8)
        t = random.uniform (-3.5, -1.5)

        pos = s*e + t*n + face.verts[i]
        yrot = math.atan2 (n.x, n.z)
        
        xform = PCD.Maths.Util.MakeXForm (pos, yrot)

        decor = obj.clone ()
        decor.applyTransformFunc (xform)

        return decor
        

    def generateContent (self, face, registry, prand):
        
        o = Object.New ('roof')
        robj = self.chooseSocketValue ('mat', registry, prand)
        o.material = robj.getObject ()
	
        verts = face.verts
        centroid = reduce (operator.add, verts) / float(len (verts))

        for i in range (len (verts)):
            ni = (i+1) % len(verts)
            t0 = (verts[i].x - centroid.x, verts[i].z - centroid.z)
            t1 = (verts[ni].x - centroid.x, verts[ni].z - centroid.z)
            o.addTri ( (verts[i], verts[ni], centroid), (t0, t1, (0,0)) )
	
        roof = MetaObject.New ()
        roof.addObjectData (o)

        if random.choice ( (True, False, False) ):
            decor = self.__genDecor (face, registry, prand)
            roof.addObjectData (decor)
        
        return roof



def Register ():
    return [ Simple() ]


