from PCD import Architecture
from PCD.Maths.Vector3D import Vec3
import random


	
class LedgeStyle (Architecture.Style.Style):
    "Style with a horizontal ledge across the front of the building"

    def __init__ (self):
	Architecture.Style.Style.__init__ (self)
        self._regname = 'basic-ledge'
        self._regsockets.append ( ('mat-ledge', '/materials/ledge') )

    def _getFeatureList (self):
        win = (random.gauss (1.3,0.2), random.gauss (1.6,0.1))
        door = (random.gauss (1.2,0.2), random.gauss (2.1,0.2))
        return [ ('window','window',win), ('door','door',door),
                 ('roof','roof', None) ]

    
    def _transformLedge (self, ledge):
        """Transforms (and possibly duplicates) the ledge object, and returns
        an Object or MetaObject"""
        ledgeheight = ledge.getHeight()
        cellheight = self.getCellSize()[1]
        ledgebase = cellheight - (ledgeheight/2)
        ledge.translate (Vec3 (0, ledgebase, 0))
        return ledge

    def _genLedge (self):
        "Generates a ledge to run across the width of the building"
        width = self.getRectSize()[0]
        registry,prand = self.getRegistry(), self.getPRand()
        fsetobj = registry.getObject ('/featureset')
        ledge = fsetobj.getObject().socketGenContent ('ledge', width,
                                                    registry, prand, 0)
        return self._transformLedge (ledge)
        

    def _startFace (self, facetype):
	if facetype == Architecture.Style.MISCFACE:
	    # no doors
	    self.doorcol = -1
	else:
	    self.doorcol = random.randrange (self.ncells[0])
            
            if facetype == Architecture.Style.FRONTFACE:
                ledge = self._genLedge ()
                self.addObject (ledge)


    def _fillCell (self, row, col, cellbase, facetype):
	if row==0 and col==self.doorcol:
	    self.addFeature ('door', cellbase)
	else:    
	    self.addFeature ('window', cellbase)
	


def Register ():
    return [ LedgeStyle() ]

	
